package com.jprofiler.api.agent.mbean;

/**
 * This is JProfiler's MBean interface. For direct calls from your code in
 * the same JVM, rather use the {@link com.jprofiler.api.agent.Controller} class.
 */
public interface RemoteControllerMBean {
    /**
     * Starts recording CPU data.
     * This method can be called repeatedly and alternatingly with
     * <tt>stopCPURecording()</tt>. With these methods you can restrict
     * CPU profiling to certain regions of your code.
     *
     * @param reset if <tt>true</tt>, any previously accumulated CPU profiling
     *              data will be discarded. If <tt>false</tt>, CPU data will be accumulated
     *              across pairs of invocations of  <tt>startCPURecording()</tt> and
     *              <tt>stopCPURecording()</tt>.
     */
    void startCPURecording(boolean reset);

    /**
     * Stops CPU recording.
     * This method can be called after <tt>startCPURecording()</tt> has been called.
     * However, you do not have to call it since CPU profiling can run until the
     * JVM exits.
     */
    void stopCPURecording();

    /**
     * Starts recording of memory allocations.
     * This method can be called repeatedly and alternatingly with
     * <tt>stopAllocRecording()</tt>. With these methods you can restrict
     * memory allocation profiling to certain regions of your code.
     * This is especially useful for profiling an application running within
     * an application server.
     *
     * @param reset if <tt>true</tt>, any previously recorded profiling
     *              data will be discarded. If <tt>false</tt>, allocations within all
     *              pairs of invocations of  <tt>startAllocRecording()</tt> and
     *              <tt>stopAllocRecording()</tt> will be recorded.
     */
    void startAllocRecording(boolean reset);

    /**
     * Stops recording of memory allocations.
     * This method can be called after <tt>startAllocRecording()</tt> has been called.
     * However, you do not have to call it since memory profiling can run until the
     * JVM exits.
     */
    void stopAllocRecording();

    /**
     * Adds a bookmark at the current time.
     * The bookmark will be displayed in all JProfiler graphs with a time axis.
     * The description will be displayed in the tooltip for the bookmark.
     *
     * @param description the name of the bookmark, may also be <tt>null</tt>
     */
    void addBookmark(String description);

    /**
     * Triggers a heap dump.
     * If you want to analyze a heap dump with the heap walker in a snapshot saved with
     * the <tt>saveSnapshot</tt> method, you should call this method from your source code
     * at an appropriate time.
     * <p/>
     * <b>ATTENTION:</b> Taking a heap dump takes a long time (on the order of seconds).
     * If you call this method to often, your application might become unusable or take
     * an excessively long time to finish.
     * </p>
     *
     * @param fullGc        if <tt>true</tt>, a full garbage collection will be performed.
     * @param onlyRecorded  if <tt>true</tt>, only objects recorded between startAllocRecording and
 *                      stopAllocRecording will be included in the dump.
     * @param primitiveData if <tt>true</tt>, primitive data will also be recorded. This has no effect for JVMTI 1.0 (Java 5).
     * @param calculateRetainedSizes if <tt>true</tt>, the retained sizes of all objects will be calculated. This requires <tt>fullGc=true</tt>.
     */
    void triggerHeapDumpWithOptions(boolean fullGc, boolean onlyRecorded, boolean primitiveData, boolean calculateRetainedSizes);

    /**
     * Triggers a heap dump.
     * Calls triggerHeapDumpWithOptions(true, false, true, true).
     * @see #triggerHeapDumpWithOptions(boolean, boolean, boolean, boolean)
     **/
    void triggerHeapDump();

    /**
     * Saves a snapshot of all profiling data to disk.
     * This is especially important for offline profiling.
     * You should choose the standard extension <tt>.jps</tt> for the
     * <tt>file</tt> parameter, since JProfiler's GUI frontend filters the corresponding
     * file choosers for that extension. If you want to save several snapshots during
     * one profiling run, please take care to provide unique <tt>file</tt> parameters
     * since snapshot files will be overwritten otherwise.
     * <p/>
     * <b>ATTENTION:</b> Saving a snapshot takes a long time (on the order of seconds).
     * If you call this method to often, your application might become unusable or take
     * an excessively long time to finish, and your hard disk might run out of space.
     * </p>
     *
     * @param file the file to which the snapshot should be saved.
     */
    void saveSnapshot(String file);

    /**
     * Starts recording of thread states and monitor usage.
     * This method can be called repeatedly and alternatingly with
     * <tt>stopThreadProfiling()</tt>. For an offline session, thread
     * profiling is switched on by default.
     */
    void startThreadProfiling();

    /**
     * Stops recording of thread states and monitor usage.
     * This method can be called repeatedly and alternatingly with
     * <tt>startThreadProfiling()</tt>. However, you do not have to call it since
     * thread profiling can run until the JVM exits.
     */
    void stopThreadProfiling();

    /**
     * Starts recording of VM telemetry data.
     * This method can be called repeatedly and alternatingly with
     * <tt>stopVMTelemetryRecording()</tt>. For an offline session, VM telemetry
     * recording is switched on by default.
     */
    void startVMTelemetryRecording();

    /**
     * Stops recording of VM telemetry data.
     * This method can be called repeatedly and alternatingly with
     * <tt>startVMTelemetryRecording()</tt>. However, you do not have to call it since
     * VM telemetry recording can run until the JVM exits.
     */
    void stopVMTelemetryRecording();

    /**
     * Saves a snapshot of all profiling data to disk when the VM shuts down.
     * This is especially important for offline profiling.
     * You should choose the standard extension <tt>.jps</tt> for the
     * <tt>file</tt> parameter, since JProfiler's GUI frontend filters the corresponding
     * file choosers for that extension.
     * <p/>
     * <b>ATTENTION:</b> Saving a snapshot can take quite some time (on the order fo seconds).
     * When the VM is shut down during a user logout or a system shutdown, the OS may terminate
     * the VM before saving is completed.
     *
     * @param file the file to which the snapshot should be saved.
     */
    void saveSnapshotOnExit(String file);


    /**
     * Enable or disable all triggers with a specified group ID. The group ID can be entered in
     * the "Group ID" step of the trigger configuration wizard in the JProfiler GUI.
     * @param enabled if the triggers should be enabled
     * @param groupId the group ID
     * @throws IllegalArgumentException if no trigger with the specified group ID exists
     */
    void enableTriggerGroup(boolean enabled, String groupId) throws IllegalArgumentException;

    /**
     * Enable or disable all triggers. The enabled/disabled state of the single triggers will not be lost,
     * disabling all triggers with this method overrides the enabled/disabled state of the single triggers.
     * @param enabled if the triggers should be enabled
     */
    void enableTriggers(boolean enabled);

    /**
     * Starts method statistics recording.
     * This will reset previously recorded method statistics.
     * This method can be called repeatedly and alternatingly with
     * <tt>stopMethodStatsRecording()</tt>.
     */
    void startMethodStatsRecording();

    /**
     * Stops method statistics recording.
     */
    void stopMethodStatsRecording();

    /**
     * Start recording of monitor usage with default thresholds of 100 microseconds
     * for blocking events and 100 ms for waiting events.
     * This method can be called repeatedly and alternatingly with
     * <tt>stopMonitorRecording()</tt>. Monitor profiling is switched off
     * by default.
     */
    public void startMonitorRecording();

    /**
     * Start recording of monitor usage.
     * This method can be called repeatedly and alternatingly with
     * <tt>stopMonitorRecording()</tt>. Monitor profiling is switched off
     * by default.
     * @param blockedThreshold the recording threshold for blocking events in microseconds.
     * @param waitingThreshold the recording threshold for waiting events in microseconds.
     */
    public void startMonitorRecording(int blockedThreshold, int waitingThreshold);

    /**
     * Stop recording of monitor usage.
     * This method can be called repeatedly and alternatingly with
     * <tt>startMonitorRecording()</tt>. However, you do not have to call it since
     * monitor profiling can run until the JVM exits.
     */
    public void stopMonitorRecording();

    /**
     * Trigger a thread dump.
     */
    void triggerThreadDump();

    void startProbeRecording(String probeName, boolean events);
    void stopProbeRecording(String probeName);

}
