package com.jprofiler.api.agent.probe;

/**
 * Base class for deriving a probe that intercepts methods. An interceptor probe can inject itself into arbitrary
 * methods and monitor entry, exit and exception exit as well as parameters and return value.
 *
 * <p>Note that implementation classes that are observed by an interceptor are only available if they are in the boot
 * classpath. Since this is usually not the case, you have to check class hierarchies by using the reflection API.
 * <p>
 * <b>Important:</b>Interceptors cannot intercept methods in the following classes:
 * <ul>
 * <li>classes that are loaded before the main class is loaded</li>
 * <li>the main class and its superclasses</li>
 * </ul>
 * <p>
 * Probes are added to the profiled process with a {@link ProbeProvider}.
 *
 * @see ProbeProvider
 */
public interface InterceptorProbe extends Probe {

    /**
     * Called when an intercepted method is entered. If you want to add payload information with a time measurement to the current call stack,
     * you should create the payload info in this method and save it in the thread local payload stack offered by the
     * interceptor context with {@link InterceptorContext#push(PayloadInfo)}. You can retrieve it again with
     * {@link InterceptorContext#pop()} in the
     * {@link #interceptionExceptionExit(InterceptorContext, Class, String, String, String, Throwable)} method
     * in order to calculate times and to publish it.
     * <p>If you perform wildcard interception based on method names only, you should check if the object has the expected
     * class hierarchy.
     *
     *
     * @param context the context you can use to create payload and probe data
     * @param object the object on which the intercepted method was called
     * @param declaringClass the class object where the intercepted method has been declared. This is <tt>null</tt> on JVMs prior to version 1.6.
     * @param declaringClassName the class where the intercepted method has been declared
     * @param methodName the name of the intercepted method
     * @param methodSignature the signature of the intercepted method in JVM format. See {@link com.jprofiler.api.agent.probe.InterceptionMethod} for more information on that format.
     * @param arguments the arguments that were passed to the intercepted method or an empty array if the method has no arguments. @see #createPayloadInfo(String)     @see #createPayloadInfo(String, Object)
     */
    void interceptionEnter(InterceptorContext context, Object object, Class declaringClass, String declaringClassName, String methodName, String methodSignature, Object[] arguments) throws Throwable;

    /**
     * Called when an intercepted method is exited via a regular return call. If you want to add payload information with
     * a time measurement to the current call stack,  you should call {@link PayloadInfo#calculateTime(InterceptorContext)} on the payload info
     * that you created in {@link #interceptionEnter(InterceptorContext, Object, Class, String, String, String, Object[])}.
     *
     * @param context the context you can use to create payload and probe data
     * @param declaringClass the class object where the intercepted method has been declared. This is <tt>null</tt> on JVMs prior to version 1.6.
     * @param declaringClassName the class where the intercepted method has been declared
     * @param methodName the name of the intercepted method
     * @param methodSignature the signature of the intercepted method in JVM format. See {@link com.jprofiler.api.agent.probe.InterceptionMethod} for more information on that format.
     * @param returnValue the value that was returned by the intercepted method or <tt>null</tt> if the return type of the method is <tt>void</tt>.
     */
    void interceptionExit(InterceptorContext context, Class declaringClass, String declaringClassName, String methodName, String methodSignature, Object returnValue) throws Throwable;

    /**
     * Called when an in intercepted method is exited because an exception is thrown.
     * @param context the context you can use to create payload and probe data
     * @param declaringClass the class object where the intercepted method has been declared. This is <tt>null</tt> on JVMs prior to version 1.6.
     * @param declaringClassName the class where the intercepted method has been declared
     * @param methodName the name of the intercepted method
     * @param methodSignature the signature of the intercepted method in JVM format. See {@link com.jprofiler.api.agent.probe.InterceptionMethod} for more information on that format.
     * @param throwable the exception that was thrown in the intercepted method
     */
    void interceptionExceptionExit(InterceptorContext context, Class declaringClass, String declaringClassName, String methodName, String methodSignature, Throwable throwable) throws Throwable;

    /**
     * Defines the methods that should be intercepted by this interceptor probe.  See {@link InterceptionMethod} for more information.
     * @return the intercepted methods
     */
    InterceptionMethod[] getInterceptionMethods();

}
